@@ -3,8 +3,9 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  1985-2018, University of Amsterdam
+    Copyright (c)  1985-2019, University of Amsterdam
                               VU University Amsterdam
+                              CWI, Amsterdam
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -83,7 +84,9 @@
             nb_setval/2,                        % +Var, +Value
             thread_create/2,                    % :Goal, -Id
             thread_join/1,                      % +Id
-            set_prolog_gc_thread/1		% +Status
+            set_prolog_gc_thread/1,		% +Status
+
+            '$wrap_predicate'/4                 % :Head, +Name, -Wrapped, +Body
           ]).
 
                 /********************************
@@ -789,6 +792,9 @@ generate_current_predicate(Name, Module, Head) :-
 :- meta_predicate
     predicate_property(:, ?).
 
+:- multifile
+    '$predicate_property'/2.
+
 :- '$iso'(predicate_property/2).
 
 predicate_property(Pred, Property) :-           % Mode ?,+
@@ -1475,3 +1481,43 @@ set_prolog_gc_thread(stop) :-
     ).
 set_prolog_gc_thread(Status) :-
     '$domain_error'(gc_thread, Status).
+
+%!  '$wrap_predicate'(:Head, +Name, -Wrapped, +Body) is det.
+%
+%   Would be nicer to have this   from library(prolog_wrap), but we need
+%   it for tabling, so it must be a system predicate.
+
+:- meta_predicate
+    '$wrap_predicate'(:, +, -, +).
+
+'$wrap_predicate'(M:Head, WName, call(Wrapped), Body) :-
+    callable_name_arguments(Head, PName, Args),
+    distinct_vars(Args, Head, Arity),
+    atomic_list_concat(['__wrap$', PName], WrapName),
+    volatile(M:WrapName/Arity),
+    WHead =.. [WrapName|Args],
+    '$c_wrap_predicate'(M:Head, WName, Wrapped, M:(WHead :- Body)).
+
+distinct_vars(Vars, _, Arity) :-
+    all_vars(Vars),
+    sort(Vars, Sorted),
+    length(Vars, Arity),
+    length(Sorted, Arity),
+    !.
+distinct_vars(_, Head, _) :-
+    '$domain_error'('most_general_term', Head).
+
+all_vars([]).
+all_vars([H|T]) :-
+    (   var(H)
+    ->  all_vars(T)
+    ;   '$uninstantiation_error'(H)
+    ).
+
+callable_name_arguments(Head, PName, Args) :-
+    atom(Head),
+    !,
+    PName = Head,
+    Args = [].
+callable_name_arguments(Head, PName, Args) :-
+    compound_name_arguments(Head, PName, Args).
