@@ -3,7 +3,8 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2012-2017, VU University Amsterdam
+    Copyright (c)  2012-2019, VU University Amsterdam
+                              CWI, Amsterdam
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -60,10 +61,7 @@
 :- use_module(library(http/http_open)).
 :- use_module(library(http/json)).
 :- use_module(library(http/http_client), []).   % plugin for POST support
-:- if(exists_source(library(archive))).
-:- use_module(library(archive)).
-:- endif.
-
+:- use_module(library(prolog_config)).
 
 /** <module> A package manager for Prolog
 
@@ -724,8 +722,9 @@ pack_install_from_local(Source, PackTopDir, Name, Options) :-
 %
 %   Unpack an archive to the given package dir.
 
-:- if(current_predicate(archive_extract/3)).
+:- if(exists_source(library(archive))).
 pack_unpack(Source, PackDir, Pack, Options) :-
+    ensure_loaded_archive,
     pack_archive_info(Source, Pack, _Info, StripOptions),
     prepare_pack_dir(PackDir, Options),
     archive_extract(Source, PackDir,
@@ -747,12 +746,21 @@ pack_unpack(_,_,_,_) :-
 %   from pack.pl in the  pack  and   Strip  is  the strip-option for
 %   archive_extract/3.
 %
+%   Requires library(archive), which is lazily loaded when needed.
+%
 %   @error  existence_error(pack_file, 'pack.pl') if the archive
 %           doesn't contain pack.pl
 %   @error  Syntax errors if pack.pl cannot be parsed.
 
-:- if(current_predicate(archive_open/3)).
+:- if(exists_source(library(archive))).
+ensure_loaded_archive :-
+    current_predicate(archive_open/3),
+    !.
+ensure_loaded_archive :-
+    use_module(library(archive)).
+
 pack_archive_info(Archive, Pack, [archive_size(Bytes)|Info], Strip) :-
+    ensure_loaded_archive,
     size_file(Archive, Bytes),
     setup_call_cleanup(
         archive_open(Archive, Handle, []),
@@ -1217,7 +1225,9 @@ def_environment('SWISOLIB', Value) :-
     current_prolog_flag(c_libplso, Value).
 def_environment('SWILIB', '-lswipl').
 def_environment('CC', Value) :-
-    (   getenv('CC', value)
+    (   getenv('CC', Value)
+    ->  true
+    ;   default_c_compiler(Value)
     ->  true
     ;   current_prolog_flag(c_cc, Value)
     ).
@@ -1236,22 +1246,24 @@ def_environment('CFLAGS', Value) :-
     atomic_list_concat([Value0, ' -I"', Home, '/include"' | Extra], Value).
 def_environment('LDSOFLAGS', Value) :-
     (   getenv('LDFLAGS', SystemFlags)
-    ->  Extra = [' ', SystemFlags|System]
+    ->  Extra = [SystemFlags|System]
     ;   Extra = System
     ),
     (   current_prolog_flag(windows, true)
     ->  current_prolog_flag(home, Home),
-        atomic_list_concat([' -L"', Home, '/bin"'], SystemLib),
+        atomic_list_concat(['-L"', Home, '/bin"'], SystemLib),
         System = [SystemLib]
-    ;   current_prolog_flag(shared_object_extension, so)
+    ;   apple_bundle_libdir(LibDir)
+    ->  atomic_list_concat(['-L"', LibDir, '"'], SystemLib),
+        System = [SystemLib]
+    ;   current_prolog_flag(c_libplso, '')
     ->  System = []                 % ELF systems do not need this
-    ;   current_prolog_flag(home, Home),
-        current_prolog_flag(arch, Arch),
-        atomic_list_concat([' -L"', Home, '/lib/', Arch, '"'], SystemLib),
+    ;   prolog_library_dir(SystemLibDir),
+        atomic_list_concat(['-L"',SystemLibDir,'"'], SystemLib),
         System = [SystemLib]
     ),
     current_prolog_flag(c_ldflags, LDFlags),
-    atomic_list_concat([LDFlags, ' -shared' | Extra], Value).
+    atomic_list_concat([LDFlags, '-shared' | Extra], ' ', Value).
 def_environment('SOEXT', Value) :-
     current_prolog_flag(shared_object_extension, Value).
 def_environment(Pass, Value) :-
@@ -1263,6 +1275,37 @@ pass_env('TEMP').
 pass_env('USER').
 pass_env('HOME').
 
+:- multifile
+    prolog:runtime_config/2.
+
+prolog_library_dir(Dir) :-
+    prolog:runtime_config(c_libdir, Dir),
+    !.
+prolog_library_dir(Dir) :-
+    current_prolog_flag(home, Home),
+    (   current_prolog_flag(c_libdir, Rel)
+    ->  atomic_list_concat([Home, Rel], /, Dir)
+    ;   current_prolog_flag(arch, Arch)
+    ->  atomic_list_concat([Home, lib, Arch], /, Dir)
+    ).
+
+%!  default_c_compiler(-CC) is semidet.
+%
+%   Try to find a  suitable  C   compiler  for  compiling  packages with
+%   foreign code.
+%
+%   @tbd Needs proper defaults for Windows.  Find MinGW?  Find MSVC?
+
+default_c_compiler(CC) :-
+    preferred_c_compiler(CC),
+    has_program(path(CC), _),
+    !.
+
+preferred_c_compiler(gcc).
+preferred_c_compiler(clang).
+preferred_c_compiler(cc).
+
+
                  /*******************************
                  *             PATHS            *
                  *******************************/
@@ -1399,9 +1442,23 @@ pack_remove(Pack) :-
     ).
 
 pack_remove_forced(Pack) :-
-    '$pack_detach'(Pack, BaseDir),
+    catch('$pack_detach'(Pack, BaseDir),
+          error(existence_error(pack, Pack), _),
+          fail),
+    !,
     print_message(informational, pack(remove(BaseDir))),
     delete_directory_and_contents(BaseDir).
+pack_remove_forced(Pack) :-
+    directory_file_path(Pack, 'pack.pl', PackFile),
+    absolute_file_name(pack(PackFile), PackPath,
+                       [ access(read),
+                         file_errors(fail)
+                       ]),
+    !,
+    file_directory_name(PackPath, BaseDir),
+    delete_directory_and_contents(BaseDir).
+pack_remove_forced(Pack) :-
+    print_message(informational, error(existence_error(pack, Pack),_)).
 
 confirm_remove(Pack, Deps, Delete) :-
     print_message(warning, pack(depends(Pack, Deps))),
