@@ -46,8 +46,8 @@
 #include <QTextBlock>
 
 #include <QTime>
+#include <QDebug>
 #include <QRegExp>
-#include <QtDebug>
 #include <QAction>
 #include <QToolTip>
 #include <QKeyEvent>
@@ -57,6 +57,8 @@
 #include <QApplication>
 #include <QStringListModel>
 
+#include "ansi_esc_seq.h"
+
 /** peek color by index */
 static QColor ANSI2col(int c, bool highlight = false) { return Preferences::ANSI2col(c, highlight); }
 
@@ -253,7 +255,7 @@ void ConsoleEdit::keyPressEvent(QKeyEvent *event) {
 
     case Key_Up:
         down = false;
-        // fall throu
+        // fall through
     case Key_Down:
         if (!ctrl) {
             // naive history handler
@@ -318,7 +320,7 @@ void ConsoleEdit::keyPressEvent(QKeyEvent *event) {
             PL_thread_raise(thids[0], SIGINT);
             return;
         }
-        // fall throu
+        // fall through
 
     default:
         accept = editable || event->matches(QKeySequence::Copy);
@@ -440,7 +442,7 @@ void ConsoleEdit::compinit2(QTextCursor c) {
     QStringList lpreds;
     foreach (auto a, atoms) {
         auto p = Completion::pred_docs.constFind(a);
-        if (p != Completion::pred_docs.end())
+        if (p != Completion::pred_docs.constEnd()) // was pred_docs.end(), seems a Qt bug it's allowed
             foreach (auto d, p.value()) {
                 QStringList la;
                 for (int n = 0; n < d.first; ++n)
@@ -506,45 +508,36 @@ void ConsoleEdit::user_output(QString text) {
     }
 
     auto instext = [&](QString text) {
-
-        int ltext;
-//#ifndef PQCONSOLE_HANDLE_HOOVERING
-#if 0
-        static QRegExp jmsg("(ERROR|Warning):[ \t]*(([a-zA-Z]:)?[^:]+):([0-9]+)(:([0-9]+))?.*", Qt::CaseSensitive, QRegExp::RegExp2);
-        if (jmsg.exactMatch(text)) {
-            QStringList parts = jmsg.capturedTexts();
-            //qDebug() << "file" << parts[2].trimmed() << "line" << parts[4].trimmed() << "char" << parts[6].trimmed();
-            auto edit = QString("'%1':%2").arg(parts[2].trimmed()).arg(parts[4].trimmed());
-            if (!parts[6].isEmpty())
-                edit += ":" + parts[6];
-            auto html = QString("<a style=\"jmsg\" href=\"system:edit(%1)\">%2</a><br>").arg(edit).arg(text);
-            //c.movePosition(c.StartOfLine);
-            c.insertHtml(html);
-            //c.movePosition(c.EndOfLine);
-        }
+        if (color_term)
+            c.insertText(text, output_text_fmt);
         else
-#endif
-        {
-            if (color_term)
-                c.insertText(text, output_text_fmt);
-            else
-                c.insertText(text);
-        }
-
+            c.insertText(text);
         if (status == wait_input) {
-            ltext = text.length();
+            int ltext = text.length();
             promptPosition += ltext;
             fixedPosition += ltext;
             ensureCursorVisible();
         }
     };
 
+    ANSI_ESC_SEQ filter(text, output_text_fmt);
+    if (filter)
+        while (filter)
+            instext(filter.next());
+    else
+        instext(text);
+
+#if 0
     // filter and apply (some) ANSI sequence
-    int pos = text.indexOf('\e');
+    int pos = text.indexOf(0x1b);
     if (pos >= 0) {
+
+//QTextStream out(stdout);
+//out << text << endl;
+
         int left = 0;
 
-        static QRegExp eseq("\e\\[(?:(3([0-7]);([01])m)|(0m)|(1m;)|1;3([0-7])m|(1m)|(?:3([0-7])m))");
+        static QRegExp eseq("\x001b\\[(?:(3([0-7]);([01])m)|(0m)|(1m;)|1;3([0-7])m|(1m)|(?:3([0-7])m))");
 
         forever {
             int pos1 = eseq.indexIn(text, pos);
@@ -596,6 +589,7 @@ void ConsoleEdit::user_output(QString text) {
     }
     else
         instext(text);
+#endif
 }
 
 bool ConsoleEdit::match_thread(int thread_id) const {
@@ -610,7 +604,7 @@ void ConsoleEdit::user_prompt(int threadId, bool tty) {
 
     is_tty = tty;
 
-    Completion::helpidx();
+    Completion::setup();
 
     QTextCursor c = textCursor();
     c.movePosition(QTextCursor::End);
@@ -708,6 +702,15 @@ void ConsoleEdit::onCursorPositionChanged() {
         setReadOnly(false);
         viewport()->setCursor(Qt::IBeamCursor);
     }
+
+    if (pmatched.size()) {
+        pmatched.format_both(c);
+        pmatched = ParenMatching::range();
+    }
+
+    ParenMatching pm(c);
+    if (pm)
+        (pmatched = pm.positions).format_both(c, pmatched.bold());
 }
 
 /** check if line content is appropriate, then highlight or open editor on it */
@@ -895,11 +898,11 @@ void ConsoleEdit::query_run(QString module, QString call) {
 }
 
 ConsoleEdit::exec_sync::exec_sync(int timeout_ms) : timeout_ms(timeout_ms) {
-    stop_ = CT;
+    stop_ = QThread::currentThread();
     go_ = 0;
 }
 void ConsoleEdit::exec_sync::stop() {
-    Q_ASSERT(CT == stop_);
+    Q_ASSERT(QThread::currentThread() == stop_);
     for ( ; ; ) {
         {   QMutexLocker lk(&sync);
             if (go_)
@@ -912,7 +915,7 @@ void ConsoleEdit::exec_sync::stop() {
 void ConsoleEdit::exec_sync::go() {
     Q_ASSERT(go_ == 0);
     Q_ASSERT(stop_ != 0);
-    auto t = CT;
+    auto t = QThread::currentThread();
     if (stop_ != t) {
         QMutexLocker lk(&sync);
         go_ = t;
