@@ -3,7 +3,7 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2011-2016, University of Amsterdam
+    Copyright (c)  2011-2017, University of Amsterdam
                               VU University Amsterdam
     All rights reserved.
 
@@ -73,12 +73,9 @@ typedef struct
 			  { FMT_ERROR("not enough arguments"); \
 			  } \
 			}
-#define FMT_ERROR(fmt)	return (void)Sunlock(fd), \
-			  PL_error(NULL, 0, NULL, ERR_FORMAT, fmt)
-#define FMT_ARG(c, a)	return (void)Sunlock(fd), \
-			       PL_error(NULL, 0, NULL, \
-					ERR_FORMAT_ARG, c, a)
-#define FMT_EXEPTION()	return (void)Sunlock(fd), FALSE
+#define FMT_ERROR(fmt)	return PL_error(NULL, 0, NULL, ERR_FORMAT, fmt)
+#define FMT_ARG(c, a)	return PL_error(NULL, 0, NULL, ERR_FORMAT_ARG, c, a)
+#define FMT_EXEPTION()	return FALSE
 
 
 static PL_locale prolog_locale =
@@ -105,7 +102,7 @@ UTF-8 format in the state's `buffer'.   The  `buffered' field represents
 the number of UTF-8 characters in the buffer.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
-static int
+static WUNUSED int
 outchr(format_state *state, int chr)
 { if ( state->pending_rubber )
   { if ( chr > 0x7f )
@@ -138,7 +135,7 @@ Emit ASCII 0-terminated strings resulting from sprintf() on numeric
 arguments.  No fuzz with wide characters here.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
-static int
+static WUNUSED int
 outstring(format_state *state, const char *s, size_t len)
 { const char *q;
   const char *e = &s[len];
@@ -160,7 +157,7 @@ outstring(format_state *state, const char *s, size_t len)
 }
 
 
-static int
+static WUNUSED int
 oututf8(format_state *state, const char *s, size_t len)
 { const char *e = &s[len];
 
@@ -176,13 +173,13 @@ oututf8(format_state *state, const char *s, size_t len)
 }
 
 
-static int
+static WUNUSED int
 oututf80(format_state *state, const char *s)
 { return oututf8(state, s, strlen(s));
 }
 
 
-static int
+static WUNUSED int
 outtext(format_state *state, PL_chars_t *txt)
 { switch(txt->encoding)
   { case ENC_ISO_LATIN_1:
@@ -315,7 +312,6 @@ format_impl(IOSTREAM *out, term_t format, term_t Args, Module m)
     PL_put_term(argv, args);
   }
 
-  startCritical;
   switch(fmt.storage)			/* format can do call-back! */
   { case PL_CHARS_RING:
     case PL_CHARS_STACK:
@@ -325,10 +321,10 @@ format_impl(IOSTREAM *out, term_t format, term_t Args, Module m)
       break;
   }
 
+  Slock(out);
   rval = do_format(out, &fmt, argc, argv, m);
+  Sunlock(out);
   PL_free_text(&fmt);
-  if ( !endCritical )
-    return FALSE;
 
   return rval;
 }
@@ -388,8 +384,6 @@ do_format(IOSTREAM *fd, PL_chars_t *fmt, int argc, term_t argv, Module m)
   unsigned int here = 0;
   int rc = TRUE;
 
-  Slock(fd);				/* buffer locally */
-
   state.out = fd;
   state.pending_rubber = 0;
   initBuffer(&state.buffer);
@@ -474,14 +468,12 @@ do_format(IOSTREAM *fd, PL_chars_t *fmt, int argc, term_t argv, Module m)
 	    tellString(&str, &bufsize, ENC_UTF8);
 	    rc = PL_call_predicate(NULL, PL_Q_PASS_EXCEPTION, proc, av);
 	    toldString();
-	    if ( !rc )
-	    { if ( str != buf )
-		free(str);
-	      goto out;
-	    }
-	    oututf8(&state, str, bufsize);
+	    if ( rc )
+	      rc = oututf8(&state, str, bufsize);
 	    if ( str != buf )
 	      free(str);
+	    if ( !rc )
+	      goto out;
 
 	    here++;
 	  } else
@@ -612,7 +604,6 @@ do_format(IOSTREAM *fd, PL_chars_t *fmt, int argc, term_t argv, Module m)
 		    { term_t r = PL_new_term_ref();
 
 		      PL_put_integer(r, arg);
-		      Sunlock(fd);
 		      return PL_error(NULL, 0, NULL, ERR_DOMAIN,
 				      ATOM_radix, r);
 		    }
@@ -672,11 +663,12 @@ do_format(IOSTREAM *fd, PL_chars_t *fmt, int argc, term_t argv, Module m)
 		    tellString(&str, &bufsize, ENC_UTF8);
 		    rc = (*f)(argv);
 		    toldString();
-		    if ( !rc )
-		      goto out;
-		    oututf8(&state, str, bufsize);
+		    if ( rc )
+		      rc = oututf8(&state, str, bufsize);
 		    if ( str != buf )
 		      free(str);
+		    if ( !rc )
+		      goto out;
 		  } else
 		  { if ( fd->position &&
 			 fd->position->linepos == state.column )
@@ -696,11 +688,12 @@ do_format(IOSTREAM *fd, PL_chars_t *fmt, int argc, term_t argv, Module m)
 		      tellString(&str, &bufsize, ENC_UTF8);
 		      rc = (*f)(argv);
 		      toldString();
-		      if ( !rc )
-		        goto out;
-		      oututf8(&state, str, bufsize);
+		      if ( rc )
+			rc = oututf8(&state, str, bufsize);
 		      if ( str != buf )
 			free(str);
+		      if ( !rc )
+			goto out;
 		    }
 		  }
 		  SHIFT;
@@ -721,11 +714,12 @@ do_format(IOSTREAM *fd, PL_chars_t *fmt, int argc, term_t argv, Module m)
 		    tellString(&str, &bufsize, ENC_UTF8);
 		    rc = (int)pl_write_term(argv, argv+1);
 		    toldString();
-		    if ( !rc )
-		      goto out;
-		    oututf8(&state, str, bufsize);
+		    if ( rc )
+		      rc = oututf8(&state, str, bufsize);
 		    if ( str != buf )
 		      free(str);
+		    if ( !rc )
+		      goto out;
 		  } else
 		  { if ( fd->position &&
 			 fd->position->linepos == state.column )
@@ -744,12 +738,13 @@ do_format(IOSTREAM *fd, PL_chars_t *fmt, int argc, term_t argv, Module m)
 		      str = buf;
 		      tellString(&str, &bufsize, ENC_UTF8);
 		      rc = (int)pl_write_term(argv, argv+1);
-		      if ( !rc )
-			goto out;
 		      toldString();
-		      oututf8(&state, str, bufsize);
+		      if ( rc )
+			rc = oututf8(&state, str, bufsize);
 		      if ( str != buf )
 			free(str);
+		      if ( !rc )
+			goto out;
 		    }
 		  }
 		  SHIFT;
@@ -762,25 +757,22 @@ do_format(IOSTREAM *fd, PL_chars_t *fmt, int argc, term_t argv, Module m)
 		  char *str = buf;
 		  size_t bufsize = BUFSIZE;
 		  term_t ex = 0;
-		  int rval;
 
 		  if ( argc < 1 )
 		  { FMT_ERROR("not enough arguments");
 		  }
 		  tellString(&str, &bufsize, ENC_UTF8);
-		  rval = callProlog(m, argv, PL_Q_CATCH_EXCEPTION, &ex);
+		  rc = callProlog(m, argv, PL_Q_CATCH_EXCEPTION, &ex);
 		  toldString();
-		  oututf8(&state, str, bufsize);
+		  if ( rc )
+		    rc = oututf8(&state, str, bufsize);
 		  if ( str != buf )
 		    free(str);
 
-		  if ( !rval )
-		  { Sunlock(fd);
-
-		    if ( ex )
-		      return PL_raise_exception(ex);
-		    else
-		      fail;
+		  if ( !rc )
+		  { if ( ex )
+		      rc = PL_raise_exception(ex);
+		    goto out;
 		  }
 
 		  SHIFT;
@@ -848,7 +840,6 @@ do_format(IOSTREAM *fd, PL_chars_t *fmt, int argc, term_t argv, Module m)
 	      default:
 	      { term_t ex = PL_new_term_ref();
 
-		Sunlock(fd);
 		PL_put_atom(ex, codeToAtom(c));
 		return PL_error("format", 2, NULL, ERR_EXISTENCE,
 				PL_new_atom("format_character"),
@@ -872,8 +863,6 @@ do_format(IOSTREAM *fd, PL_chars_t *fmt, int argc, term_t argv, Module m)
     emit_rubber(&state);
 
 out:
-  Sunlock(fd);
-
   return rc;
 }
 
@@ -1601,6 +1590,10 @@ formatFloat(PL_locale *locale, int how, int arg, Number f, Buffer out)
 	  return NULL;
 	}
 	written = snprintf(baseBuffer(out, char), size, tmp, f->value.f);
+#ifdef __WINDOWS__
+        if ( written < 0 )	/* pre-C99 Windows snprintf() returns -1 */
+	  written = size*2;
+#endif
       }
 
 #ifdef __WINDOWS__
