@@ -338,7 +338,9 @@ static void
 setClauseChoice(ClauseChoice chp, ClauseRef cref, gen_t generation ARG_LD)
 { while ( cref &&
 	  cref->value.clause->generation.erased <= generation )
-    cref = cref->next;
+  { cref = cref->next;
+    LD->clauses.erased_skipped++;
+  }
 
   chp->cref = cref;
 }
@@ -1203,7 +1205,7 @@ static int
 has_pow2_clauses(Definition def)
 { unsigned int nc = def->impl.clauses.number_of_clauses;
 
-  return 1<<MSB(nc) == nc;
+  return nc > 0 && 1<<MSB(nc) == nc;
 }
 
 
@@ -1569,8 +1571,7 @@ the new index to the indexes of the predicate.
 
 static ClauseIndex
 hashDefinition(ClauseList clist, hash_hints *hints, IndexContext ctx)
-{ GET_LD
-  ClauseRef cref;
+{ ClauseRef cref;
   ClauseIndex ci;
   ClauseIndex *cip;
 
@@ -1580,8 +1581,10 @@ hashDefinition(ClauseList clist, hash_hints *hints, IndexContext ctx)
 			  iargsName(hints->args, NULL), 2<<hints->ln_buckets,
 			  hints->list ? "lists" : "clauses"));
 
-#ifdef O_PLMT
+#if defined(O_PLMT) && !defined(NDEBUG)
+{ GET_LD
   assert(LD->thread.info->access.predicate == ctx->predicate);
+}
 #endif
 
   canonicalHap(hints->args);
