@@ -79,6 +79,7 @@ typedef struct tbl_component
   component_set        *children;		/* Child components */
   worklist_set         *worklist;		/* Worklist of current query */
   worklist_set         *created_worklists;	/* Worklists created */
+  trie		       *leader;			/* Leading variant */
 } tbl_component;
 
 
@@ -97,20 +98,22 @@ typedef struct worklist
 { cluster      *head;			/* answer and dependency clusters */
   cluster      *tail;
   cluster      *riac;			/* rightmost inner answer cluster */
+  cluster      *free_clusters;		/* clusters to reuse */
   int		magic;			/* WORKLIST_MAGIC */
   unsigned	ground : 1;		/* Ground call (early completion) */
   unsigned	executing : 1;		/* $tbl_wkl_work/3 in progress */
   unsigned	in_global_wl : 1;	/* already in global worklist */
   unsigned	negative : 1;		/* this is a suspended negation */
   unsigned	neg_delayed : 1;	/* Negative node was delayed */
-  unsigned	neg_completed : 1;	/* Negative node is complete (false) */
   unsigned	has_answers : 1;	/* Negative node has >= one answer */
+  unsigned	answer_completed : 1;	/* Is answer completed */
   size_t	undefined;		/* #undefined answers */
 
   tbl_component*component;		/* component I belong to */
   trie	       *table;			/* My answer table */
 
   buffer	delays;			/* Delayed answers */
+  buffer	pos_undefined;		/* Positive undefined */
 } worklist;
 
 
@@ -151,6 +154,7 @@ typedef struct delay
 typedef struct delay_set
 { unsigned	     offset;		/* offset in delays */
   unsigned	     size;		/* size of the conjunction */
+  unsigned	     active;		/* active members of conjunction */
 } delay_set;
 
 typedef struct delay_info
